@@ -20,6 +20,7 @@
 
 #include <api/ShareApi.h>
 
+#include <web-server/HttpManager.h>
 #include <web-server/Session.h>
 #include <web-server/WebServerManager.h>
 
@@ -29,35 +30,42 @@
 #include <api/common/Validation.h>
 
 #include <web-server/JsonUtil.h>
+#include <web-server/WebServerSettings.h>
 
-#include <airdcpp/ClientManager.h>
-#include <airdcpp/HashManager.h>
-#include <airdcpp/HubEntry.h>
-#include <airdcpp/Magnet.h>
-#include <airdcpp/SearchResult.h>
-#include <airdcpp/ShareManager.h>
-#include <airdcpp/SharePathValidator.h>
-#include <airdcpp/StringTokenizer.h>
+#include <airdcpp/hub/ClientManager.h>
+#include <airdcpp/hash/HashManager.h>
+#include <airdcpp/favorites/HubEntry.h>
+#include <airdcpp/core/classes/Magnet.h>
+#include <airdcpp/util/PathUtil.h>
+#include <airdcpp/search/SearchQuery.h>
+#include <airdcpp/search/SearchResult.h>
+#include <airdcpp/share/ShareManager.h>
+#include <airdcpp/share/SharePathValidator.h>
+#include <airdcpp/util/text/StringTokenizer.h>
+#include <airdcpp/share/temp_share/TempShareManager.h>
+
+
+#define HOOK_FILE_VALIDATION "share_file_validation_hook"
+#define HOOK_DIRECTORY_VALIDATION "share_directory_validation_hook"
+
+#define HOOK_NEW_FILE_VALIDATION "new_share_file_validation_hook"
+#define HOOK_NEW_DIRECTORY_VALIDATION "new_share_directory_validation_hook"
 
 namespace webserver {
-	ShareApi::ShareApi(Session* aSession) : 
-		HookApiModule(
-			aSession, 
-			Access::SETTINGS_VIEW, 
-			{
-				"share_refresh_queued",
-				"share_refresh_started",
-				"share_refresh_completed",
-				
-				"share_exclude_added",
-				"share_exclude_removed",
+	ShareApi::ShareApi(Session* aSession) : HookApiModule(aSession, Access::SETTINGS_VIEW, Access::SETTINGS_EDIT) {
+		createSubscriptions({
+			"share_refresh_queued",
+			"share_refresh_started",
+			"share_refresh_completed",
 
-				"share_temp_item_added",
-				"share_temp_item_removed",
-			},
-			Access::SETTINGS_EDIT
-		) 
-	{
+			"share_exclude_added",
+			"share_exclude_removed",
+
+			"share_temp_item_added",
+			"share_temp_item_removed",
+		});
+
+		// Methods
 		METHOD_HANDLER(Access::ANY,				METHOD_GET,		(EXACT_PARAM("grouped_root_paths")),				ShareApi::handleGetGroupedRootPaths);
 		METHOD_HANDLER(Access::SETTINGS_VIEW,	METHOD_GET,		(EXACT_PARAM("stats")),								ShareApi::handleGetStats);
 		METHOD_HANDLER(Access::ANY,				METHOD_POST,	(EXACT_PARAM("find_dupe_paths")),					ShareApi::handleFindDupePaths);
@@ -65,6 +73,12 @@ namespace webserver {
 		METHOD_HANDLER(Access::ANY,				METHOD_POST,	(EXACT_PARAM("validate_path")),						ShareApi::handleValidatePath);
 		METHOD_HANDLER(Access::ANY,				METHOD_POST,	(EXACT_PARAM("check_path_shared")),					ShareApi::handleIsPathShared);
 
+		METHOD_HANDLER(Access::SETTINGS_VIEW,	METHOD_POST,	(EXACT_PARAM("directories"), EXACT_PARAM("by_real"), EXACT_PARAM("content"), RANGE_START_PARAM, RANGE_MAX_PARAM),	ShareApi::handleGetDirectoryContentByReal);
+		METHOD_HANDLER(Access::SETTINGS_VIEW,	METHOD_POST,	(EXACT_PARAM("directories"), EXACT_PARAM("by_real")),		ShareApi::handleGetDirectoryByReal);
+		METHOD_HANDLER(Access::SETTINGS_VIEW,	METHOD_POST,	(EXACT_PARAM("files"), EXACT_PARAM("by_real")),				ShareApi::handleGetFileByReal);
+		METHOD_HANDLER(Access::SETTINGS_VIEW,	METHOD_GET,		(EXACT_PARAM("files"), TTH_PARAM),							ShareApi::handleGetFilesByTTH);
+
+
 		METHOD_HANDLER(Access::SETTINGS_EDIT,	METHOD_POST,	(EXACT_PARAM("refresh")),							ShareApi::handleRefreshShare);
 		METHOD_HANDLER(Access::SETTINGS_EDIT,	METHOD_DELETE,	(EXACT_PARAM("refresh")),							ShareApi::handleAbortRefreshShare);
 		METHOD_HANDLER(Access::SETTINGS_EDIT,	METHOD_POST,	(EXACT_PARAM("refresh"), EXACT_PARAM("paths")),		ShareApi::handleRefreshPaths);
@@ -81,38 +95,13 @@ namespace webserver {
 		METHOD_HANDLER(Access::SETTINGS_EDIT,	METHOD_POST,	(EXACT_PARAM("temp_shares")),						ShareApi::handleAddTempShare);
 		METHOD_HANDLER(Access::SETTINGS_EDIT,	METHOD_DELETE,	(EXACT_PARAM("temp_shares"), TOKEN_PARAM),			ShareApi::handleRemoveTempShare);
 
-		createHook("share_file_validation_hook", [this](ActionHookSubscriber&& aSubscriber) {
-			return ShareManager::getInstance()->getValidator().fileValidationHook.addSubscriber(std::move(aSubscriber), HOOK_HANDLER(ShareApi::fileValidationHook));
-		}, [this](const string& aId) {
-			ShareManager::getInstance()->getValidator().fileValidationHook.removeSubscriber(aId);
-		}, [this] {
-			return ShareManager::getInstance()->getValidator().fileValidationHook.getSubscribers();
-		});
-
-		createHook("share_directory_validation_hook", [this](ActionHookSubscriber&& aSubscriber) {
-			return ShareManager::getInstance()->getValidator().directoryValidationHook.addSubscriber(std::move(aSubscriber), HOOK_HANDLER(ShareApi::directoryValidationHook));
-		}, [this](const string& aId) {
-			ShareManager::getInstance()->getValidator().directoryValidationHook.removeSubscriber(aId);
-		}, [this] {
-			return ShareManager::getInstance()->getValidator().directoryValidationHook.getSubscribers();
-		});
-
-		createHook("new_share_directory_validation_hook", [this](ActionHookSubscriber&& aSubscriber) {
-			return ShareManager::getInstance()->getValidator().newDirectoryValidationHook.addSubscriber(std::move(aSubscriber), HOOK_HANDLER(ShareApi::newDirectoryValidationHook));
-		}, [this](const string& aId) {
-			ShareManager::getInstance()->getValidator().newDirectoryValidationHook.removeSubscriber(aId);
-		}, [this] {
-			return ShareManager::getInstance()->getValidator().newDirectoryValidationHook.getSubscribers();
-		});
-
-		createHook("new_share_file_validation_hook", [this](ActionHookSubscriber&& aSubscriber) {
-			return ShareManager::getInstance()->getValidator().newFileValidationHook.addSubscriber(std::move(aSubscriber), HOOK_HANDLER(ShareApi::newFileValidationHook));
-		}, [this](const string& aId) {
-			ShareManager::getInstance()->getValidator().newFileValidationHook.removeSubscriber(aId);
-		}, [this] {
-			return ShareManager::getInstance()->getValidator().newFileValidationHook.getSubscribers();
-		});
+		// Hooks
+		HOOK_HANDLER(HOOK_FILE_VALIDATION,			ShareManager::getInstance()->getValidator().fileValidationHook,			ShareApi::fileValidationHook);
+		HOOK_HANDLER(HOOK_DIRECTORY_VALIDATION,		ShareManager::getInstance()->getValidator().directoryValidationHook,	ShareApi::directoryValidationHook);
+		HOOK_HANDLER(HOOK_NEW_FILE_VALIDATION,		ShareManager::getInstance()->getValidator().newFileValidationHook,		ShareApi::newFileValidationHook);
+		HOOK_HANDLER(HOOK_NEW_DIRECTORY_VALIDATION, ShareManager::getInstance()->getValidator().newDirectoryValidationHook, ShareApi::newDirectoryValidationHook);
 
+		// Listeners
 		ShareManager::getInstance()->addListener(this);
 	}
 
@@ -122,7 +111,7 @@ namespace webserver {
 
 	ActionHookResult<> ShareApi::fileValidationHook(const string& aPath, int64_t aSize, const ActionHookResultGetter<>& aResultGetter) noexcept {
 		return HookCompletionData::toResult(
-			fireHook("share_file_validation_hook", WEBCFG(SHARE_FILE_VALIDATION_HOOK_TIMEOUT).num(), [&]() {
+			maybeFireHook(HOOK_FILE_VALIDATION, WEBCFG(SHARE_FILE_VALIDATION_HOOK_TIMEOUT).num(), [&]() {
 				return json({
 					{ "path", aPath },
 					{ "size", aSize },
@@ -134,7 +123,7 @@ namespace webserver {
 
 	ActionHookResult<> ShareApi::directoryValidationHook(const string& aPath, const ActionHookResultGetter<>& aResultGetter) noexcept {
 		return HookCompletionData::toResult(
-			fireHook("share_directory_validation_hook", WEBCFG(SHARE_DIRECTORY_VALIDATION_HOOK_TIMEOUT).num(), [&]() {
+			maybeFireHook(HOOK_DIRECTORY_VALIDATION, WEBCFG(SHARE_DIRECTORY_VALIDATION_HOOK_TIMEOUT).num(), [&]() {
 				return json({
 					{ "path", aPath },
 				});
@@ -145,7 +134,7 @@ namespace webserver {
 
 	ActionHookResult<> ShareApi::newFileValidationHook(const string& aPath, int64_t aSize, bool aNewParent, const ActionHookResultGetter<>& aResultGetter) noexcept {
 		return HookCompletionData::toResult(
-			fireHook("new_share_file_validation_hook", WEBCFG(NEW_SHARE_FILE_VALIDATION_HOOK_TIMEOUT).num(), [&]() {
+			maybeFireHook(HOOK_NEW_FILE_VALIDATION, WEBCFG(NEW_SHARE_FILE_VALIDATION_HOOK_TIMEOUT).num(), [&]() {
 				return json({
 					{ "path", aPath },
 					{ "size", aSize },
@@ -158,7 +147,7 @@ namespace webserver {
 
 	ActionHookResult<> ShareApi::newDirectoryValidationHook(const string& aPath, bool aNewParent, const ActionHookResultGetter<>& aResultGetter) noexcept {
 		return HookCompletionData::toResult(
-			fireHook("new_share_directory_validation_hook", WEBCFG(NEW_SHARE_DIRECTORY_VALIDATION_HOOK_TIMEOUT).num(), [&]() {
+			maybeFireHook(HOOK_NEW_DIRECTORY_VALIDATION, WEBCFG(NEW_SHARE_DIRECTORY_VALIDATION_HOOK_TIMEOUT).num(), [&]() {
 				return json({
 					{ "path", aPath },
 					{ "new_parent", aNewParent },
@@ -168,8 +157,51 @@ namespace webserver {
 		);
 	}
 
-	json ShareApi::serializeShareItem(const SearchResultPtr& aSR) noexcept {
-		auto isDirectory = aSR->getType() == SearchResult::TYPE_DIRECTORY;
+	json ShareApi::serializeShareItem(const ShareItem& aItem) noexcept {
+		if (aItem.directory) {
+			return serializeDirectory(aItem.directory);
+		} else {
+			return serializeFile(aItem.file);
+		}
+	}
+
+	json ShareApi::serializeFile(const ShareDirectory::File* aFile) noexcept {
+		auto realPath = aFile->getRealPath();
+		return {
+			{ "id", ValueGenerator::generatePathId(realPath) },
+			{ "name", aFile->getName().getNormal() },
+			{ "path", realPath },
+			{ "virtual_path", aFile->getAdcPath() },
+			{ "size", aFile->getSize() },
+			{ "tth", aFile->getTTH().toBase32() },
+			{ "time", aFile->getLastWrite() },
+			{ "type", Serializer::serializeFileType(aFile->getName().getLower()) },
+			{ "profiles", aFile->getParent()->getRootProfiles() },
+		};
+	}
+
+	json ShareApi::serializeDirectory(const ShareDirectory::Ptr& aDirectory) noexcept {
+		auto contentInfo(DirectoryContentInfo::empty());
+		int64_t totalSize = 0;
+		aDirectory->getContentInfo(totalSize, contentInfo);
+
+		auto realPath = aDirectory->getRealPathUnsafe();
+		return {
+			{ "id", ValueGenerator::generatePathId(realPath) },
+			{ "name", aDirectory->getRealName().getNormal() },
+			{ "path", realPath },
+			{ "virtual_path", aDirectory->getAdcPathUnsafe() },
+			{ "size", totalSize },
+			{ "tth", Util::emptyString },
+			{ "time", aDirectory->getLastWrite() },
+			{ "type", Serializer::serializeFolderType(contentInfo) },
+			{ "profiles", aDirectory->getRootProfiles() },
+		};
+	}
+
+
+	json ShareApi::serializeVirtualItem(const SearchResultPtr& aSR) noexcept {
+		auto isDirectory = aSR->getType() == SearchResult::Type::DIRECTORY;
 		auto path = aSR->getAdcPath();
 
 		StringList realPaths;
@@ -191,7 +223,7 @@ namespace webserver {
 		};
 	}
 
-	json ShareApi::serializeRefreshQueueInfo(const ShareManager::RefreshTaskQueueInfo& aRefreshQueueInfo) noexcept {
+	json ShareApi::serializeRefreshQueueInfo(const RefreshTaskQueueInfo& aRefreshQueueInfo) noexcept {
 		return {
 			{ "task", !aRefreshQueueInfo.token ? JsonUtil::emptyJson : json({
 				{ "id", json(*aRefreshQueueInfo.token) },
@@ -212,11 +244,11 @@ namespace webserver {
 		};
 	}
 
-	string ShareApi::refreshResultToString(ShareManager::RefreshTaskQueueResult aRefreshQueueResult) noexcept {
+	string ShareApi::refreshResultToString(RefreshTaskQueueResult aRefreshQueueResult) noexcept {
 		switch (aRefreshQueueResult) {
-			case ShareManager::RefreshTaskQueueResult::EXISTS: return "exists";
-			case ShareManager::RefreshTaskQueueResult::QUEUED: return "queued";
-			case ShareManager::RefreshTaskQueueResult::STARTED: return "started";
+			case RefreshTaskQueueResult::EXISTS: return "exists";
+			case RefreshTaskQueueResult::QUEUED: return "queued";
+			case RefreshTaskQueueResult::STARTED: return "started";
 		}
 
 		dcassert(0);
@@ -240,20 +272,73 @@ namespace webserver {
 
 		// Parse share profile and query
 		auto profile = Deserializer::deserializeOptionalShareProfile(reqJson);
-		auto s = FileSearchParser::parseSearch(reqJson, true, Util::toString(Util::rand()));
+		auto s = FileSearchParser::parseSearch(reqJson, true);
 
 		// Search
 		SearchResultList results;
 		
 		{
-			unique_ptr<SearchQuery> matcher(SearchQuery::getSearch(s));
+			unique_ptr<SearchQuery> matcher(SearchQuery::fromSearch(s));
+			ShareSearch search(*matcher, profile, nullptr, s->path);
 			try {
-				ShareManager::getInstance()->adcSearch(results, *matcher, profile, CID(), s->path);
+				ShareManager::getInstance()->search(results, search);
 			} catch (...) {}
 		}
 
 		// Serialize results
-		aRequest.setResponseBody(Serializer::serializeList(results, serializeShareItem));
+		aRequest.setResponseBody(Serializer::serializeList(results, serializeVirtualItem));
+		return websocketpp::http::status_code::ok;
+	}
+
+	api_return ShareApi::handleGetFilesByTTH(ApiRequest& aRequest) {
+		auto tth = aRequest.getTTHParam();
+
+		const auto files = ShareManager::getInstance()->findFiles(tth);
+		aRequest.setResponseBody(Serializer::serializeList(files, serializeFile));
+		return websocketpp::http::status_code::ok;
+	}
+
+	api_return ShareApi::handleGetDirectoryContentByReal(ApiRequest& aRequest) {
+		auto path = JsonUtil::getField<string>("path", aRequest.getRequestBody(), false);
+		auto start = aRequest.getRangeParam(START_POS);
+		auto count = aRequest.getRangeParam(MAX_COUNT);
+		if (!ShareManager::getInstance()->findDirectoryByRealPath(path, [&](const ShareDirectory::Ptr& aDirectory) {
+			ShareItem::List items;
+			for (const auto& d : aDirectory->getDirectories()) {
+				items.emplace_back(d);
+			}
+			for (const auto& f : aDirectory->getFiles()) {
+				items.emplace_back(f);
+			}
+
+			auto j = Serializer::serializeFromPosition(start, count, items, serializeShareItem);
+			aRequest.setResponseBody(j);
+		})) {
+			JsonUtil::throwError("path", JsonException::ERROR_INVALID, "Path was not found");
+		}
+
+		return websocketpp::http::status_code::ok;
+	}
+
+	api_return ShareApi::handleGetFileByReal(ApiRequest& aRequest) {
+		auto path = JsonUtil::getField<string>("path", aRequest.getRequestBody(), false);
+		if (!ShareManager::getInstance()->findFileByRealPath(path, [&](const ShareDirectory::File& aFile) {
+			aRequest.setResponseBody(serializeFile(&aFile));
+		})) {
+			JsonUtil::throwError("path", JsonException::ERROR_INVALID, "Path was not found");
+		}
+
+		return websocketpp::http::status_code::ok;
+	}
+
+	api_return ShareApi::handleGetDirectoryByReal(ApiRequest& aRequest) {
+		auto path = JsonUtil::getField<string>("path", aRequest.getRequestBody(), false);
+		if (!ShareManager::getInstance()->findDirectoryByRealPath(path, [&](const ShareDirectory::Ptr& aDirectory) {
+			aRequest.setResponseBody(serializeDirectory(aDirectory));
+		})) {
+			JsonUtil::throwError("path", JsonException::ERROR_INVALID, "Path was not found");
+		}
+
 		return websocketpp::http::status_code::ok;
 	}
 
@@ -263,9 +348,9 @@ namespace webserver {
 		const auto user = Deserializer::deserializeUser(aRequest.getRequestBody(), true, true);
 		const auto optionalClient = Deserializer::deserializeClient(aRequest.getRequestBody(), true);
 
-		const auto filePath = aRequest.getSession()->getServer()->getFileServer().getTempFilePath(fileId);
-		if (filePath.empty() || !Util::fileExists(filePath)) {
-			JsonUtil::throwError("file_id", JsonUtil::ERROR_INVALID, "Source file was not found");
+		const auto filePath = aRequest.getSession()->getServer()->getHttpManager().getFileServer().getTempFilePath(fileId);
+		if (filePath.empty() || !PathUtil::fileExists(filePath)) {
+			JsonUtil::throwError("file_id", JsonException::ERROR_INVALID, "Source file was not found");
 		}
 
 		const auto size = File::getSize(filePath);
@@ -285,7 +370,7 @@ namespace webserver {
 		}
 
 		auto shareProfileToken = optionalClient ? optionalClient->get(HubSettings::ShareProfile) : SETTING(DEFAULT_SP);
-		auto item = ShareManager::getInstance()->addTempShare(tth, name, filePath, size, shareProfileToken, user);
+		auto item = TempShareManager::getInstance()->addTempShare(tth, name, filePath, size, shareProfileToken, user);
 
 		aRequest.setResponseBody({
 			{ "magnet", Magnet::makeMagnet(tth, name, size) },
@@ -297,7 +382,7 @@ namespace webserver {
 
 	api_return ShareApi::handleRemoveTempShare(ApiRequest& aRequest) {
 		auto token = aRequest.getTokenParam();
-		if (!ShareManager::getInstance()->removeTempShare(token)) {
+		if (!TempShareManager::getInstance()->removeTempShare(token)) {
 			aRequest.setResponseErrorStr("Temp share item " + Util::toString(token) + " was not found");
 			return websocketpp::http::status_code::bad_request;
 		}
@@ -309,7 +394,7 @@ namespace webserver {
 		return {
 			{ "id", aInfo.id },
 			{ "name", aInfo.name },
-			{ "path", aInfo.path },
+			{ "path", aInfo.realPath },
 			{ "size", aInfo.size },
 			{ "tth", aInfo.tth.toBase32() },
 			{ "time_added", aInfo.timeAdded },
@@ -319,7 +404,7 @@ namespace webserver {
 	}
 
 	api_return ShareApi::handleGetTempShares(ApiRequest& aRequest) {
-		const auto tempShares = ShareManager::getInstance()->getTempShares();
+		const auto tempShares = TempShareManager::getInstance()->getTempShares();
 
 		aRequest.setResponseBody(Serializer::serializeList(tempShares, serializeTempShare));
 		return websocketpp::http::status_code::ok;
@@ -331,7 +416,7 @@ namespace webserver {
 	}
 
 	api_return ShareApi::handleAddExclude(ApiRequest& aRequest) {
-		auto path = JsonUtil::getField<string>("path", aRequest.getRequestBody(), false);
+		auto path = PathUtil::validateDirectoryPath(JsonUtil::getField<string>("path", aRequest.getRequestBody(), false));
 
 		try {
 			ShareManager::getInstance()->addExcludedPath(path);
@@ -346,7 +431,7 @@ namespace webserver {
 	api_return ShareApi::handleRemoveExclude(ApiRequest& aRequest) {
 		auto path = JsonUtil::getField<string>("path", aRequest.getRequestBody(), false);
 		if (!ShareManager::getInstance()->removeExcludedPath(path)) {
-			JsonUtil::throwError("path", JsonUtil::ERROR_INVALID, "Excluded path was not found");
+			JsonUtil::throwError("path", JsonException::ERROR_INVALID, "Excluded path was not found");
 		}
 
 		return websocketpp::http::status_code::no_content;
@@ -365,13 +450,13 @@ namespace webserver {
 	}
 
 
-	void ShareApi::on(ShareManagerListener::TempFileAdded, const TempShareInfo& aFile) noexcept {
+	void ShareApi::on(TempShareManagerListener::TempFileAdded, const TempShareInfo& aFile) noexcept {
 		maybeSend("share_temp_item_added", [&] {
 			return serializeTempShare(aFile);
 		});
 	}
 
-	void ShareApi::on(ShareManagerListener::TempFileRemoved, const TempShareInfo& aFile) noexcept {
+	void ShareApi::on(TempShareManagerListener::TempFileRemoved, const TempShareInfo& aFile) noexcept {
 		maybeSend("share_temp_item_removed", [&] {
 			return serializeTempShare(aFile);
 		});
@@ -411,12 +496,12 @@ namespace webserver {
 	api_return ShareApi::handleRefreshPaths(ApiRequest& aRequest) {
 		const auto& reqJson = aRequest.getRequestBody();
 		addAsyncTask([
-			paths = JsonUtil::getField<StringList>("paths", reqJson, false),
+			paths = Deserializer::deserializeList<string>("paths", reqJson, Deserializer::directoryPathArrayValueParser, false),
 			priority = parseRefreshPriority(reqJson),
 			complete = aRequest.defer(),
 			callerPtr = aRequest.getOwnerPtr()
 		] {
-			ShareManager::RefreshTaskQueueInfo refreshInfo;
+			RefreshTaskQueueInfo refreshInfo;
 			const auto refreshF = [&] {
 				refreshInfo = ShareManager::getInstance()->refreshPathsHookedThrow(priority, paths, callerPtr);
 			};
@@ -567,7 +652,7 @@ namespace webserver {
 		auto path = JsonUtil::getOptionalField<string>("path", reqJson);
 		if (path) {
 			// Note: non-standard/partial paths are allowed, no strict directory path validation
-			ret = ShareManager::getInstance()->getAdcDirectoryPaths(*path);
+			ret = ShareManager::getInstance()->getAdcDirectoryDupePaths(*path);
 		} else {
 			auto tth = Deserializer::deserializeTTH(reqJson);
 			ret = ShareManager::getInstance()->getRealPaths(tth);
@@ -601,7 +686,7 @@ namespace webserver {
 			return ShareRefreshPriority::MANUAL;
 		}
 
-		JsonUtil::throwError("priority_type", JsonUtil::ERROR_INVALID, "Refresh priority " + priority + "doesn't exist");
+		JsonUtil::throwError("priority_type", JsonException::ERROR_INVALID, "Refresh priority " + priority + "doesn't exist");
 		return ShareRefreshPriority::NORMAL;
 	}
 
