@@ -19,13 +19,13 @@
 #ifndef DCPLUSPLUS_DCPP_FILESEARCH_PARSER_H
 #define DCPLUSPLUS_DCPP_FILESEARCH_PARSER_H
 
-#include <airdcpp/typedefs.h>
-#include <airdcpp/Search.h>
+#include <airdcpp/core/header/typedefs.h>
+#include <airdcpp/search/Search.h>
 
 namespace webserver {
 	class FileSearchParser {
 	public:
-		static SearchPtr parseSearch(const json& aJson, bool aIsDirectSearch, const string& aToken);
+		static SearchPtr parseSearch(const json& aJson, bool aIsDirectSearch);
 
 		static string parseSearchType(const string& aType);
 		static string serializeSearchType(const string& aType);
