@@ -21,44 +21,53 @@
 #include <web-server/ExtensionManager.h>
 #include <web-server/Extension.h>
 #include <web-server/NpmRepository.h>
+#include <web-server/Session.h>
+#include <web-server/SocketManager.h>
 #include <web-server/TarFile.h>
 #include <web-server/WebServerManager.h>
+#include <web-server/WebServerSettings.h>
 #include <web-server/WebSocket.h>
 
-#include <airdcpp/CryptoManager.h>
-#include <airdcpp/Encoder.h>
-#include <airdcpp/File.h>
-#include <airdcpp/HttpDownload.h>
-#include <airdcpp/LogManager.h>
-#include <airdcpp/ScopedFunctor.h>
-#include <airdcpp/SimpleXML.h>
-#include <airdcpp/StringTokenizer.h>
-#include <airdcpp/Thread.h>
-#include <airdcpp/TimerManager.h>
-#include <airdcpp/UpdateManager.h>
-#include <airdcpp/ZUtils.h>
+#include <airdcpp/util/CryptoUtil.h>
+#include <airdcpp/hash/value/Encoder.h>
+#include <airdcpp/core/classes/Exception.h>
+#include <airdcpp/core/io/File.h>
+#include <airdcpp/connection/http/HttpDownload.h>
+#include <airdcpp/events/LogManager.h>
+#include <airdcpp/util/PathUtil.h>
+#include <airdcpp/core/classes/ScopedFunctor.h>
+#include <airdcpp/core/io/xml/SimpleXML.h>
+#include <airdcpp/util/text/StringTokenizer.h>
+#include <airdcpp/util/SystemUtil.h>
+#include <airdcpp/core/thread/Thread.h>
+#include <airdcpp/core/timer/TimerManager.h>
+#include <airdcpp/core/update/UpdateManager.h>
+#include <airdcpp/core/io/compress/ZUtils.h>
 
 
 namespace webserver {
 	ExtensionManager::ExtensionManager(WebServerManager* aWsm) : wsm(aWsm) {
 		wsm->addListener(this);
+		wsm->getSocketManager().addListener(this);
 
 		npmRepository = make_unique<NpmRepository>(
-			std::bind(&ExtensionManager::downloadExtension, this, placeholders::_1, placeholders::_2, placeholders::_3),
-			std::bind(&ExtensionManager::log, this, placeholders::_1, placeholders::_2)
+			std::bind_front(&ExtensionManager::downloadExtension, this),
+			std::bind_front(&ExtensionManager::log, this)
 		);
 	}
 
 	ExtensionManager::~ExtensionManager() {
+		wsm->getSocketManager().removeListener(this);
 		wsm->removeListener(this);
 	}
 
 	void ExtensionManager::log(const string& aMsg, LogMessage::Severity aSeverity) const noexcept {
-		// wsm->log(aMsg, aSeverity);
 		LogManager::getInstance()->message(aMsg, aSeverity, STRING(EXTENSIONS));
 	}
 
 	void ExtensionManager::on(WebServerManagerListener::Started) noexcept {
+		wsm->getSocketManager().addListener(this);
+
 		// Don't add in the constructor as core may not have been initialized at that point
 		UpdateManager::getInstance()->addListener(this);
 
@@ -77,9 +86,11 @@ namespace webserver {
 				ext->removeListeners();
 
 				if (!ext->isManaged()) {
+					// Handle unmanaged extensions in WebServerManagerListener::Stopped (they could still connect at this point as the server is running)
 					continue;
 				}
-
+				
+				// Managed extensions can't be started again after this
 				try {
 					ext->stopThrow();
 				} catch (const Exception& e) {
@@ -88,6 +99,8 @@ namespace webserver {
 			}
 		}
 
+		wsm->getSocketManager().removeListener(this);
+
 		UpdateManager::getInstance()->removeListener(this);
 		fire(ExtensionManagerListener::Stopped());
 	}
@@ -104,24 +117,33 @@ namespace webserver {
 			Thread::sleep(50);
 		}
 
+		ExtensionList unmanagedExtensions;
+		{
+			RLock l(cs);
+			ranges::copy_if(extensions, back_inserter(unmanagedExtensions), [](const ExtensionPtr& e) { return !e->isManaged(); });
+		}
+
+		for (const auto& e: unmanagedExtensions) {
+			unregisterRemoteExtension(e);
+		}
+
 		WLock l(cs);
 		dcassert(ranges::all_of(extensions, [](const ExtensionPtr& aExtension) { return !aExtension->getSession(); }));
 		extensions.clear();
 	}
 
-	void ExtensionManager::on(WebServerManagerListener::SocketDisconnected, const WebSocketPtr& aSocket) noexcept {
+	void ExtensionManager::on(SocketManagerListener::SocketDisconnected, const WebSocketPtr& aSocket) noexcept {
 		if (!aSocket->getSession()) {
 			return;
 		}
 
-		const auto session = aSocket->getSession();
-		aSocket->getSession()->getServer()->addAsyncTask([session, this] {
+		aSocket->getSession()->getServer()->addAsyncTask([session = aSocket->getSession(), this] {
 			ExtensionPtr extension = nullptr;
 
 			// Remove possible unmanaged extensions matching this session
 			{
 				RLock l(cs);
-				auto i = find_if(extensions.begin(), extensions.end(), [&](const ExtensionPtr& aExtension) {
+				auto i = ranges::find_if(extensions, [&](const ExtensionPtr& aExtension) {
 					return aExtension->getSession() == session;
 				});
 
@@ -166,7 +188,7 @@ namespace webserver {
 
 					{
 						WLock l(cs);
-						blockedExtensions.emplace(name, reason);
+						blockedExtensions.try_emplace(name, reason);
 					}
 				}
 
@@ -192,10 +214,10 @@ namespace webserver {
 			}
 		}
 
-		for (const auto& blockedInfo: toRemove) {
+		for (const auto& [ext, message] : toRemove) {
 			try {
-				log(STRING_F(WEB_EXTENSION_UNINSTALL_BLOCKED, blockedInfo.first->getName() % blockedInfo.second), LogMessage::SEV_WARNING);
-				uninstallLocalExtensionThrow(blockedInfo.first, true);
+				log(STRING_F(WEB_EXTENSION_UNINSTALL_BLOCKED, ext->getName() % message), LogMessage::SEV_WARNING);
+				uninstallLocalExtensionThrow(ext, true);
 			} catch (const Exception& e) {
 				log(e.what(), LogMessage::SEV_ERROR);
 			}
@@ -209,7 +231,7 @@ namespace webserver {
 		int started = 0;
 		for (const auto& path : directories) {
 			auto ext = loadLocalExtension(path);
-			if (ext && startExtensionImpl(ext, engines)) {
+			if (ext && !ext->isDisabled() && startExtensionImpl(ext, engines)) {
 				started++;
 			}
 		}
@@ -276,7 +298,7 @@ namespace webserver {
 	bool ExtensionManager::removeExtension(const ExtensionPtr& aExtension) noexcept {
 		{
 			WLock l(cs);
-			auto i = find(extensions.begin(), extensions.end(), aExtension);
+			auto i = ranges::find(extensions, aExtension);
 			if (i != extensions.end()) {
 				extensions.erase(i);
 			} else {
@@ -299,7 +321,7 @@ namespace webserver {
 			aExtension->stopThrow();
 		} catch (const Exception& e) {
 			if (!aForced) {
-				throw e;
+				throw;
 			}
 
 			// Try to continue in any case...
@@ -328,15 +350,34 @@ namespace webserver {
 		fire(ExtensionManagerListener::InstallationStarted(), aInstallId);
 
 		WLock l(cs);
-		auto ret = httpDownloads.emplace(aUrl, make_shared<HttpDownload>(aUrl, [=, this]() {
+		auto [_, added] = httpDownloads.try_emplace(aUrl, make_shared<HttpDownload>(aUrl, [aInstallId, aUrl, aSha1, this]() {
 			onExtensionDownloadCompleted(aInstallId, aUrl, aSha1);
 		}));
 
-		return ret.second;
+		return added;
+	}
+
+	bool ExtensionManager::validateSha1(const string& aData, const string& aSha1) noexcept {
+		if (aSha1.empty()) {
+			return true;
+		}
+
+		auto calculatedSha1 = CryptoUtil::calculateSha1(aData);
+		if (calculatedSha1) {
+			char mdString[SHA_DIGEST_LENGTH * 2 + 1];
+			for (int i = 0; i < SHA_DIGEST_LENGTH; i++)
+				snprintf(&mdString[i * 2], sizeof(mdString), "%02x", (*calculatedSha1)[i]);
+
+			if (compare(string(mdString), aSha1) == 0) {
+				return true;
+			}
+		}
+
+		return false;
 	}
 
 	void ExtensionManager::onExtensionDownloadCompleted(const string& aInstallId, const string& aUrl, const string& aSha1) noexcept {
-		auto tempFile = Util::getPath(Util::PATH_TEMP) + Util::validateFileName(aUrl) + ".tmp";
+		auto tempFile = AppUtil::getPath(AppUtil::PATH_TEMP) + PathUtil::validateFileName(aUrl) + ".tmp";
 
 		// Don't allow the same download to be initiated again until the installation has finished
 		ScopedFunctor([&]() {
@@ -367,18 +408,9 @@ namespace webserver {
 			}
 
 			// Validate the possible checksum
-			if (!aSha1.empty()) {
-				auto calculatedSha1 = CryptoManager::calculateSha1(download->buf);
-				if (calculatedSha1) {
-					char mdString[SHA_DIGEST_LENGTH * 2 + 1];
-					for (int i = 0; i < SHA_DIGEST_LENGTH; i++)
-						snprintf(&mdString[i * 2], sizeof(mdString), "%02x", (*calculatedSha1)[i]);
-
-					if (compare(string(mdString), aSha1) != 0) {
-						failInstallation(aInstallId, STRING(WEB_EXTENSION_DOWNLOAD_FAILED), STRING(WEB_EXTENSION_CHECKSUM_MISMATCH));
-						return;
-					}
-				}
+			if (!validateSha1(download->buf, aSha1)) {
+				failInstallation(aInstallId, STRING(WEB_EXTENSION_DOWNLOAD_FAILED), STRING(WEB_EXTENSION_CHECKSUM_MISMATCH));
+				return;
 			}
 
 			// Save on disk
@@ -408,7 +440,7 @@ namespace webserver {
 			return;
 		}
 
-		string tempRoot = Util::getPath(Util::PATH_TEMP) + "extension_" + Util::getFileName(aInstallFilePath) + PATH_SEPARATOR_STR;
+		string tempRoot = AppUtil::getPath(AppUtil::PATH_TEMP) + "extension_" + PathUtil::getFileName(aInstallFilePath) + PATH_SEPARATOR_STR;
 		ScopedFunctor([&tempRoot]() {
 			try {
 				File::removeDirectoryForced(tempRoot);
@@ -463,7 +495,9 @@ namespace webserver {
 
 		// Updating an existing extension?
 		auto extension = getExtension(extensionName);
+		auto wasStopped = false;
 		if (extension) {
+			wasStopped = !extension->isRunning();
 			if (!extension->isManaged()) {
 				failInstallation(aInstallId, STRING(WEB_EXTENSION_EXISTS), "Unmanaged extensions can't be upgraded");
 				return;
@@ -478,16 +512,16 @@ namespace webserver {
 			}
 
 			try {
-				File::removeDirectoryForced(Util::joinDirectory(extension->getRootPath(), EXT_PACKAGE_DIR));
+				File::removeDirectoryForced(PathUtil::joinDirectory(extension->getRootPath(), EXT_PACKAGE_DIR));
 			} catch (const FileException& e) {
-				failInstallation(aInstallId, "Failed to remove the old extension package directory " + Util::joinDirectory(extension->getRootPath(), EXT_PACKAGE_DIR), e.getError());
+				failInstallation(aInstallId, "Failed to remove the old extension package directory " + PathUtil::joinDirectory(extension->getRootPath(), EXT_PACKAGE_DIR), e.getError());
 				return;
 			}
 		}
 
 		try {
 			// Move files to final destination directory
-			File::moveDirectory(tempPackageDirectory, Util::joinDirectory(Extension::getRootPath(extensionName), EXT_PACKAGE_DIR));
+			File::moveDirectory(tempPackageDirectory, PathUtil::joinDirectory(Extension::getRootPath(extensionName), EXT_PACKAGE_DIR));
 		} catch (const FileException& e) {
 			failInstallation(aInstallId, "Failed to move extension files to the final destination directory", e.what());
 			return;
@@ -518,7 +552,10 @@ namespace webserver {
 			log(STRING_F(WEB_EXTENSION_INSTALLED, extension->getName()), LogMessage::SEV_INFO);
 		}
 
-		startExtensionImpl(extension, getEngines());
+		if (!wasStopped) {
+			startExtensionImpl(extension, getEngines());
+		}
+
 		fire(ExtensionManagerListener::InstallationSucceeded(), aInstallId, extension, updated);
 	}
 
@@ -587,8 +624,8 @@ namespace webserver {
 	void ExtensionManager::onExtensionFailed(const Extension* aExtension, uint32_t aExitCode) noexcept {
 		if (aExitCode == EXIT_CODE_TIMEOUT || aExitCode == EXIT_CODE_IO_ERROR || aExitCode == EXIT_CODE_TEMP_ERROR) {
 			// Attempt to restart it (but outside of extension's timer thread)
-			auto name = aExtension->getName();
-			wsm->addAsyncTask([=, this] {
+			const auto& name = aExtension->getName();
+			wsm->addAsyncTask([aExtension, name, this] {
 				// Wait for the log file handles to get closed
 				Thread::sleep(3000);
 
@@ -617,8 +654,8 @@ namespace webserver {
 		ExtensionPtr ext = nullptr;
 		try {
 			ext = std::make_shared<Extension>(
-				Util::joinDirectory(aPath, EXT_PACKAGE_DIR),
-				std::bind(&ExtensionManager::onExtensionFailed, this, std::placeholders::_1, std::placeholders::_2)
+				PathUtil::joinDirectory(aPath, EXT_PACKAGE_DIR),
+				std::bind_front(&ExtensionManager::onExtensionFailed, this)
 			);
 		} catch (const Exception& e) {
 			log(STRING_F(WEB_EXTENSION_LOAD_ERROR_X, aPath % e.what()), LogMessage::SEV_ERROR);
@@ -657,7 +694,7 @@ namespace webserver {
 		string lastError;
 		for (const auto& supportedExtEngine: aSupportedExtEngines) {
 			// Find an installed engine that can run this extension
-			auto engineIter = find_if(aInstalledEngines.begin(), aInstalledEngines.end(), [&supportedExtEngine](const auto& e) { return e.name == supportedExtEngine; });
+			auto engineIter = ranges::find_if(aInstalledEngines, [&supportedExtEngine](const auto& e) { return e.name == supportedExtEngine; });
 			if (engineIter == aInstalledEngines.end()) {
 				lastError = STRING_F(WEB_EXTENSION_ENGINE_NO_CONFIG, supportedExtEngine);
 				continue;
@@ -685,13 +722,13 @@ namespace webserver {
 		for (const auto& token: tokens.getTokens()) {
 			if (File::isAbsolutePath(token)) {
 				// Full path
-				if (Util::fileExists(token)) {
+				if (PathUtil::fileExists(token)) {
 					return token;
 				}
 			} else if (token.length() >= 2 && token.compare(0, 2, "./") == 0) {
 				// Relative path
-				auto fullPath = Util::getAppFilePath() + token.substr(2);
-				if (Util::fileExists(fullPath)) {
+				auto fullPath = AppUtil::getAppFilePath() + token.substr(2);
+				if (PathUtil::fileExists(fullPath)) {
 					return fullPath;
 				}
 			} else {
@@ -703,7 +740,7 @@ namespace webserver {
 #endif
 				testCommand += " " + token;
 
-				if (Util::runSystemCommand(testCommand) == 0) {
+				if (dcpp::SystemUtil::runSystemCommand(testCommand) == 0) {
 					return token;
 				}
 			}
