@@ -30,7 +30,7 @@ namespace webserver {
 
 		// CallbackWrapper is meant to ensure the lifetime of the timer
 		// (which necessary only if the timer is called from a class that can be deleted, such as sessions)
-		Timer(Callback&& aCallback, boost::asio::io_service& aIO, time_t aIntervalMillis, const CallbackWrapper& aWrapper) :
+		Timer(Callback&& aCallback, boost::asio::io_context& aIO, time_t aIntervalMillis, const CallbackWrapper& aWrapper) :
 			cb(std::move(aCallback)),
 			cbWrapper(aWrapper),
 			timer(aIO),
